<?php
/**
 * Twitter image discoverer and embedder
 *
 *	Takes a Twitter message, searches for an embedded image, and
 *	replaces the URL to the image with an image tag, wrapped in a div tag,
 *	and scaled to thumbnail size.  Currently supports Tapulous's Twinkle and
 *	Twitpic.
 *
 *	Caches results in a database table for speed.
 *
 *	Usage:
 *		$message = TwitterImageUtil::processContent($message, 
 *													$cacheDatabaseHandle,
 *													$options);
 *
 *		Options: An associative array with specified options. Defaults are:
 *
 *					divClass				= "twitter_image"
 *					thumbnailDimensions		= array(35,35)
 *					thumbnailDirectory		= "twitter-thumbs"
 *					thumbnailURL			= "twitter-thumbs"
 *					cacheTable				= "twitter_image_cache"
 *					lightboxMode			= false
 *
 *
 *	Copyright 2008-2009 Michael Tyson <mike@tyson.id.au>, Tzi Software
 *	Version 0.2.6, 22 September 2009
 *	GPL License
 */

//define("DEBUG_MODE", true);

function DEBUG($str) {
    if ( !defined("DEBUG_MODE") || !DEBUG_MODE ) return;
    
    global $_debug_fd;
    if ( !$_debug_fd ) {
        $_debug_fd = fopen("twitter-debug.log", "a");
    }
    fwrite($_debug_fd, $str."\n");
    fflush($_debug_fd);
}

$GLOBALS["__TwitterImageUtil_defaults"] = array(
		"divClass"				=> "twitter_image", 
		"thumbnailDimensions"	=> array(35,35), 
		"thumbnailDirectory"	=> "twitter-thumbs",
		"thumbnailURL"			=> "twitter-thumbs", 
		"cacheTable"			=> "twitter_image_cache",
		"lightboxMode"			=> false);

class TwitterImageUtil {

	/**
	 * Add a plugin function
	 *
	 *	Plugin functions search content, passed as an argument, and
	 *	return an associative array with 'imageUrl', 'linkUrl', 'replaceStart', and 'replaceLength' elements.
	 *
	 * @param	callback	Callback	Function taking one argument and returning above associative array
	 */
	function registerPlugin($callback) {
		global $twitterImageUtilPlugins;
		$twitterImageUtilPlugins[] = $callback;
	}
	
	function processContent($content, 
							$cacheDatabaseHandle,
							$options = array()) {

        $content = strip_tags($content);
 	    $contentId = "'".trim(substr($content, 0, 20))."...'";
	   
		$options = array_merge($GLOBALS["__TwitterImageUtil_defaults"], $options);
	    
		$id = md5($content);
	   
		// Try to find cached version
		if ( ($info = TwitterImageUtil::findInCache($cacheDatabaseHandle, $options["cacheTable"], $id)) !== false ) {
		   
			if ( !$info ) {
				// No image here
				DEBUG("Cached version found for content $contentId; no image here");
				return $content;
			}
		   
			// See if thumbnail exists
			if ( file_exists($options["thumbnailDirectory"]."/".$id.".jpg") ) {
				// Thumbnail's there - we're done
				DEBUG("Cached version found for content $contentId; found image");
				return $info["content"];
			}
		}
	   
	   
		if ( !$info ) {
			// No cached version exists. Try to discover a image in the content
			if ( !($info = TwitterImageUtil::findTwitterImageFromContent($content)) ) {
			    
			    DEBUG("Content $contentId processed; no image here");
			    
				// Didn't find a image. Cache result
				TwitterImageUtil::saveCache($cacheDatabaseHandle, $options["cacheTable"], $id, $info);
			   
				// Return content as-is
				return $content;
			}
		}
	    
	    DEBUG("Content $contentId processed; found image at ".$info["imageUrl"]);
	    
		// Found an image; create a thumbnail
		if ( TwitterImageUtil::createThumbnail($id, $info /* In/Out param */, $options["thumbnailDimensions"], $options["thumbnailDirectory"]) ) {
			// Succeeded - create url to thumbnail
			$thumbUrl = $options["thumbnailURL"]."/".$id.".jpg";
			DEBUG("Made thumbnail $id.jpg for content $contentId''");
		} else {
			// Failed making thumbnail - just link to original
			$thumbUrl = $info["imageUrl"];
			$info["thumbnailDimensions"] = $options["thumbnailDimensions"];
			DEBUG("Failed making thumbnail for content $contentId");
		}
	   
		// Create tag
		$alt = trim(htmlspecialchars(strip_tags(substr_replace($content, '', $info['replaceStart'], $info['replaceLength']))));
		$tag = "<div class=\"".$options["divClass"]."\">".
				(($options["lightboxMode"] && $info["lightbox"])  ?
						"<a href=\"".$info["imageUrl"]."\" rel=\"lightbox[twitter]\" title=\"".$alt."\">" 
					: 	"<a href=\"".$info["linkUrl"]."\" title=\"".$alt."\">").
				"<img src=\"".$thumbUrl."\" ".($info["thumbnailDimensions"] ? "width=\"".$info["thumbnailDimensions"][0]."\" height=\"".$info["thumbnailDimensions"][1]."\"" : "").
					" alt=\"".$alt."\" />".
				"</a></div>";
			   
		// Insert tag into content
		$info["content"] = substr_replace($content, $tag, $info['replaceStart'], $info['replaceLength']);
	   
		// Save to cache
		TwitterImageUtil::saveCache($cacheDatabaseHandle, $options["cacheTable"], $id, $info);
	   
	    DEBUG("Finished processing $contentId. Result:\n".print_r($info, true));
	   
		return $info["content"];
	}



	function findTwitterImageFromContent($content) {
		global $twitterImageUtilPlugins;
		
		DEBUG("Running ".(is_array($twitterImageUtilPlugins) ? count($twitterImageUtilPlugins) : 0)." plugins");
		
		if ( is_array($twitterImageUtilPlugins) )
		foreach ( $twitterImageUtilPlugins as $callback ) {
			$info = $callback($content);
			if ( $info ) return $info;
		}
	   
		return false;
	}
   
	function createThumbnail($id, &$info, $thumbnailDimensions, $thumbnailDirectory) {
	   
		if ( !file_exists($thumbnailDirectory) ) {
			mkdir($thumbnailDirectory, 0755);
		}
	   
		if ( !file_exists($thumbnailDirectory) ) {
			return false;
		}
	   
		$output = $thumbnailDirectory."/".$id.".jpg";
	   
		if ( file_exists($output) ) {
			return true;
		}

		$extension = strtolower(substr($info["imageUrl"], (($pos=strrpos($info["imageUrl"], ".")) !== false ? $pos : strlen($info["imageUrl"]))));
		if ( strlen($extension) != 3 ) $extension = "";
		
		if ( !function_exists("imagecreatefromjpeg") ) {
			// GD2 is not installed: try downloading the image, then using imagemagick's convert
			if ( !($fd = @fopen($info["imageUrl"], "r")) ) {
				// Couldn't download
				return false;
			}
		   
			$tmpImagePath = "TwitterImage-tmp.".($extension ? $extension : "jpg");
		   
			if ( !($ofd = fopen($tmpImagePath, "w")) ) {
				// Couldn't open output image
				return false;
			}
		   
			while ( !feof($fd) )
				fwrite($ofd, fgets($fd));
			fclose($ofd);
			fclose($fd);
		   
			if ( trim(shell_exec("convert ".escapeshellarg($tmpImagePath)." -resize ".escapeshellarg($thumbnailDimensions[0]."x".$thumbnailDimensions[1])." ".escapeshellarg($output)." && echo success")) != "success" ||
				!file_exists($output) ) {
				@unlink($output);
				return false;
			}
			@unlink($tmpImagePath);
		   
			return true;
		}
	    
		if ( $extension == "png" )
			$image = @imagecreatefrompng($info["imageUrl"]);
		else if ( $extension == "gif" )
			$image = @imagecreatefromgif($info["imageUrl"]);
		else {
			// assume jpeg
			$image = @imagecreatefromjpeg($info["imageUrl"]);
			if ( !$image ) {
				// try png
				$image = @imagecreatefrompng($info["imageUrl"]);
			}
		}
	
		if ( !$image ) {
			// Failed opening the image
			return false;
		}
	   
		$scaledImage = TwitterImageUtil::scaleImage($image, $thumbnailDimensions);
	   
		if ( !$scaledImage ) {
			// Failed scaling the image
			return false;
		}
	   
		$info["thumbnailDimensions"] = array(imagesx($scaledImage), imagesy($scaledImage));
	   

		if ( !imagejpeg($scaledImage, $output) ) {
			return false;
		}
	   
		imagedestroy($image);
		imagedestroy($scaledImage);
	   
		return true;
	}
	
	function scaleImage($image, $dimensions)
	{
		// Read original image dimensions
		$orig_w=imagesx($image);
		$orig_h=imagesy($image);
	   
		list($width,$height) = $dimensions;

		// Scale proportionately to within height/width
		if ($orig_w>$width || $orig_h>$height)
		{
		   $img_w=$width;
		   $img_h=$height;
		   if ($img_w/$orig_w*$orig_h>$img_h)
			  $img_w=round($img_h*$orig_w/$orig_h);
		   else
			  $img_h=round($img_w*$orig_h/$orig_w);
		} 
		else
		{
		   $img_w=$orig_w;
		   $img_h=$orig_h;
		}
	   
		// Create scaled image
		$newImage = imagecreatetruecolor($img_w,$img_h);

		if ( !$newImage )
		   return false;

		imagecopyresampled($newImage,$image,0,0,0,0,$img_w,$img_h,$orig_w,$orig_h);
		
		return $newImage;
	}
   
	function findInCache($cacheDatabaseHandle, $cacheTable, $id) {
		$result = mysql_query("SELECT * FROM `".$cacheTable."` WHERE identifier = '".mysql_escape_string($id)."'");
		if ( !$result || !mysql_num_rows($result) ) return false;
	   
		$obj = mysql_fetch_object($result);
		$info = @unserialize($obj->info);
	   
		if ( !$info ) 
			$info = array();
		if ( $obj->content )
			$info["content"] = $obj->content;
	   
		return $info;
	}
   
	function saveCache($cacheDatabaseHandle, $cacheTable, $id, $info) {
		if ( !TwitterImageUtil::createCache($cacheDatabaseHandle, $cacheTable) )
			return false;
	   
		return mysql_query(
			"INSERT INTO `".$cacheTable."` (`identifier`, `content`, `info`) VALUES (".
				"'".mysql_escape_string($id)."',".
				"'".mysql_escape_string($info["content"])."',".
				"'".mysql_escape_string(serialize($info))."')", $cacheDatabaseHandle);
	}
	
	
	function clearCache($cacheDatabaseHandle, $cacheTable=false, $thumbnailDirectory=false) {
		if ( !$cacheTable ) $cacheTable = $GLOBALS["__TwitterImageUtil_defaults"]["cacheTable"];
		if ( !$thumbnailDirectory ) $thumbnailDirectory = $GLOBALS["__TwitterImageUtil_defaults"]["thumbnailDirectory"];
		if ( file_exists($thumbnailDirectory) ) {
			if ( ($dir=@opendir($thumbnailDirectory)) )
			while ( ($file = readdir($dir)) ) {
				if ( preg_match('|^[0-9a-z]+\.jpg$|', $file) )
					@unlink($thumbnailDirectory."/".$file);
			}
			@rmdir($thumbnailDirectory);
		}
		return mysql_query("DROP TABLE `".$cacheTable."`", $cacheDatabaseHandle);
	}
   
	function createCache($cacheDatabaseHandle, $cacheTable) {
		return mysql_query(
			"CREATE TABLE IF NOT EXISTS `".$cacheTable."` (
			  `id` int(11) NOT NULL auto_increment,
			  `identifier` varchar(32) default NULL,
			  `content` text,
			  `info` text,
			  PRIMARY KEY  (`id`),
			  UNIQUE KEY `identifier` (`identifier`)
			) ENGINE=MyISAM DEFAULT CHARSET=latin1",
			$cacheDatabaseHandle);
	}
}


/**
 * Begin support for various image sources
 */

// Twinkle
function twinkle_image_support($content) {
	if ( !preg_match("|http://snipurl.com/([0-9a-zA-Z]+)|", $content, $matches, PREG_OFFSET_CAPTURE) ) {
	    DEBUG("No 'snipurl' URL found in content for Twinkle");
		return false;
	}

	$snipId = $matches[1][0];

	if ( !($fd = @fopen("http://snipurl.com/resolveurl?id=".$snipId, "r")) ) {
	    DEBUG("Couldn't load snipurl page to resolve id $snipId");
		return false;
	}

	$url = "";
	while ( !feof($fd) )
		$url .= fgets($fd);
	fclose($fd);

	if ( !preg_match("|http://twinkle.tapulous.com/index.php\?hash=[0-9a-zA-Z]+|", $url) ) {
	    DEBUG("Couldn't find twinkle.tapulous.com redirect for snipurl id $snipId");
		return false;
	}
   
	if ( !($fd = @fopen(trim($url), "r")) ) {
	    DEBUG("Couldn't load twinkle page");
		return false;
	}
   
	$data = "";
	while ( !feof($fd) )
		$data .= fgets($fd);
	fclose($fd);

	if ( !preg_match("@<img[^>]+?src=\"(http://[^\"]+.jpg)\"@", $data, $datamatches) ) {
	    DEBUG("Couldn't locate an image in data on twinkle page");
		return;
	}

	$info = array();
	$info["imageUrl"] = $datamatches[1];
	$info["linkUrl"] = $matches[0][0];
	$info["replaceStart"] = $matches[0][1];
	$info["replaceLength"] = strlen($matches[0][0]);
	$info["lightbox"] = true;
	
	DEBUG("Successfully located a twinkle image");
	
	return $info;
}

TwitterImageUtil::registerPlugin('twinkle_image_support');

// Twitpic
function twitpic_image_support($content) {
	
	if ( !preg_match("|http://twitpic.com/([0-9a-zA-Z]+)|", $content, $matches, PREG_OFFSET_CAPTURE) ) {
	    DEBUG("Couldn't find twitpic URL in content");
		return false;
	}

	$url = $matches[0][0];

	if ( !($fd = @fopen($url."/full", "r")) ) {
	    DEBUG("Couldn't load twitpic page");
		return false;
	}

	$data = "";
	while ( !feof($fd) )
		$data .= fgets($fd);
	fclose($fd);

	if ( !preg_match('|<img class="photo-large"[^>]+src="([^"]+)|', $data, $datamatches) ) {
	    DEBUG("Couldn't find image with id 'pic' in twitpic page content");
		return false;
	}
   
	$info = array();
	$info["imageUrl"] = $datamatches[1];
	$info["linkUrl"] = $url;
	$info["replaceStart"] = $matches[0][1];
	$info["replaceLength"] = strlen($matches[0][0]);
	$info["lightbox"] = true;
	
	DEBUG("Successfully located a twitpic image");
	
	return $info;
}

TwitterImageUtil::registerPlugin('twitpic_image_support');




?>